# -*- coding: iso-8859-1 -*-
"""
    getfiles - Obtain files using MoinMessage

    @copyright: 2013, 2014 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMessage import *
from email.mime.application import MIMEApplication
from email.mime.text import MIMEText
from email.parser import Parser
from os.path import join, exists
from os import makedirs
import sys

def writefile(filename, s):
    f = open(filename, "wb")
    try:
        f.write(s)
    finally:
        f.close()

if __name__ == "__main__":
    try:
        service = sys.argv[1]
        signer = sys.argv[2]
        url = sys.argv[3]
        sender = sys.argv[4]
        datetime = sys.argv[5]
        target_dir = sys.argv[6]
    except IndexError:
        print >>sys.stderr, """\
Need a service key identifier, signing key identifier, the repository URL,
sender identifier, the datetime of the last resources retrieved, and the target
directory as arguments to this program.
"""
        sys.exit(1)

    # Bundle files into a message.

    message = Message()

    part = MIMEText("\n".join([sender, datetime]), "moinmessage-request")
    message.add_update(part)

    # Get the e-mail message itself.

    message = message.get_payload()

    # Encrypt, sign and send the message.

    gpg = GPG()
    message = gpg.encryptMessage(message, service)
    message = gpg.signMessage(message, signer)
    resp = sendMessageForReading(message, url)

    # Verify, decrypt and unpack the message.

    try:
        if not is_signed(message):
            print >>sys.stderr, "Incoming message was not signed."
            sys.exit(1)

        message = Parser().parse(resp)
        fingerprint, identity, content = gpg.verifyMessage(message)

        if is_encrypted(content):
            text = gpg.decryptMessage(content)
            content = Parser().parsestr(text)

    except MoinMessageDecodingError:
        print >>sys.stderr, "Incoming message was improperly encoded."
        sys.exit(1)

    except MoinMessageError, exc:
        print >>sys.stderr, "Incoming message was not verifiable: %s" % exc
        sys.exit(1)

    message = Message()
    message.handle_message(content)

    for part in message.updates:

        # Use the "outer" filename to determine a directory for the retrieved
        # file, even though the eventual filename in the directory may be
        # different.

        filename = part["Content-Disposition"]
        directory, leafname = filename.split("/")

        # The data may be encrypted.

        try:
            data = gpg.decryptMessage(part)
        except MoinMessageError:
            print >>sys.stderr, "Message part was not decrypted."
            data = part.get_payload(decode=True)

        # Parse the decoded data.

        content = Parser().parsestr(data)

        if content.is_multipart():
            files = content.get_payload()
        else:
            files = [content]

        # Treat each part of the verified, possibly encrypted message as a file.

        for file in files:
            data = file.get_payload(decode=True)

            realname = file.get("Content-Disposition")
            filename = join(directory, realname or leafname)

            dirpath = join(target_dir, directory)
            if not exists(dirpath):
                makedirs(dirpath)
            writefile(join(target_dir, filename), data)

# vim: tabstop=4 expandtab shiftwidth=4
